@@ -1,6 +1,7 @@
 """Module that contains Theta Client class."""
-import os
-import warnings
+import re
+import shutil
+import subprocess
 from decimal import Decimal
 from threading import Thread
 from time import sleep
@@ -21,7 +22,7 @@ from .parsing import (
 )
 from .terminal import check_download, launch_terminal
 
-_NOT_CONNECTED_MSG = "You must esetablish a connection first."
+_NOT_CONNECTED_MSG = "You must establish a connection first."
 
 
 def _format_strike(strike: float) -> int:
@@ -34,17 +35,33 @@ def _format_date(dt: date) -> str:
     return dt.strftime("%Y%m%d")
 
 
+def _verify_java():
+    if not shutil.which("java"):
+        print('Java 11 or higher is required to use this API. Please install Java on this machine.')
+        exit(1)
+    version = subprocess.check_output(['java', '-version'], stderr=subprocess.STDOUT)
+    pattern = '\"(\d+\.\d+).*\"'
+    version = float(re.search(pattern, version.decode('utf8')).groups()[0])
+
+    if version < 11:
+        print('Java 11 or higher is required to use this API. You are using using Java '
+              + version + '. Please upgrade to a newer version.')
+        exit(1)
+
+
+# noinspection GrazieInspection
 class ThetaClient:
     """A high-level, blocking client used to fetch market data."""
 
-    def __init__(self, port: int = 11000, timeout: Optional[float] = 60, launch: bool = True,
+    def __init__(self, port: int = 11000, timeout: Optional[float] = 60, launch: bool = True, jvm_mem: int = 0,
                  username: str = "default", passwd: str = "default", auto_update: bool = True, use_bundle: bool = False):
         """Construct a client instance to interface with market data.
 
         :param port: The port number specified in the Theta Terminal config
         :param timeout: The max number of seconds to wait for a response before
             throwing a TimeoutError
-        :param launch: If true, launches the terminal; False uses an existing external terminal instance.
+        :param launch: Launches the terminal if true; uses an existing external terminal instance if false.
+        :jvm_mem: Any integer provided above zero will force the terminal to allocate a maximum amount of memory in GB.
         :param username: Theta Data username / email. When specified with a
             password, this class will launch the terminal in headless mode.
         :param passwd: Theta Data password. When specified with a username,
@@ -57,9 +74,9 @@ class ThetaClient:
         self.port: int = port
         self.timeout = timeout
         self._server: Optional[socket.socket] = None  # None while disconnected
-        if os.system("java -version") != 0:
-            warnings.warn("Java 11 is required to use this API. Terminating...")
-            return
+        self.launch = launch
+
+        _verify_java()
 
         if launch:
             if passwd.__contains__('%') or passwd.__contains__('?') or passwd.__contains__(' '):
@@ -67,11 +84,11 @@ class ThetaClient:
                                       'Please change it at https://thetadata.net -> login -> forgot password.')
             if username == "default" or passwd == "default":
                 print("You are using the free version of Theta Data. You are currently limited to "
-                      "20 requests / minute. A data subscription can be purchased at https://thetadata.net."
+                      "20 requests / minute. A data subscription can be purchased at https://thetadata.net. "
                       "If you already have a Theta Data account, specify the username and passwd parameters.")
 
             check_download(auto_update)
-            Thread(target=launch_terminal, args=[username, passwd, use_bundle]).start()
+            Thread(target=launch_terminal, args=[username, passwd, use_bundle, jvm_mem]).start()
         else:
             print("You are not launching the terminal. This means you should have an external instance already running.")
 
@@ -96,8 +113,11 @@ class ThetaClient:
                                               ' Try restarting your system.')
                     sleep(1)
             self._server.settimeout(self.timeout)
+            self._send_ver()
             yield
         finally:
+            if self.launch:
+                self.kill()
             self._server.close()
 
     def _recv(self, n_bytes: int, progress_bar: bool = False) -> bytearray:
@@ -109,7 +129,7 @@ class ThetaClient:
         assert self._server is not None, _NOT_CONNECTED_MSG
 
         # receive body data in parts
-        MAX_PART_SIZE = 512  # 4 KiB recommended for most machines
+        MAX_PART_SIZE = 256  # 4 KiB recommended for most machines
 
         buffer = bytearray(n_bytes)
         bytes_downloaded = 0
@@ -118,23 +138,37 @@ class ThetaClient:
         range_ = range(0, n_bytes, MAX_PART_SIZE)
         iterable = tqdm(range_, desc="Downloading") if progress_bar else range_
 
+        start = 0
         for i in iterable:
             part_size = min(MAX_PART_SIZE, n_bytes - bytes_downloaded)
             bytes_downloaded += part_size
             part = self._server.recv(part_size)
-            buffer[i : i + part_size] = part
+            start += 1
+            buffer[i: i + part_size] = part
 
         assert bytes_downloaded == n_bytes
         return buffer
 
-    def kill(self) -> None:
+    def _send_ver(self):
+        ver_msg = f"MSG_CODE={MessageType.HIST.value}&version=0.5.6\n"
+        self._server.sendall(ver_msg.encode("utf-8"))
+
+    def kill(self, ignore_err=True) -> None:
         """Remotely kill the Terminal process.
 
         All subsequent requests will timeout until the Terminal is restarted.
         """
-        # assert self._server is not None, _NOT_CONNECTED_MSG We should be able to kill the terminal no matter what
+        if not ignore_err:
+            assert self._server is not None, _NOT_CONNECTED_MSG
+
         kill_msg = f"MSG_CODE={MessageType.KILL.value}\n"
-        self._server.sendall(kill_msg.encode("utf-8"))
+        try:
+            self._server.sendall(kill_msg.encode("utf-8"))
+        except OSError:
+            if ignore_err:
+                pass
+            else:
+                raise OSError
 
     def get_hist_option(
         self,
@@ -145,6 +179,7 @@ class ThetaClient:
         right: OptionRight,
         date_range: DateRange,
         interval_size: int = 0,
+        use_rth: bool = False,
         progress_bar: bool = False,
     ) -> pd.DataFrame:
         """
@@ -156,7 +191,8 @@ class ThetaClient:
         :param strike:         The strike price in USD, rounded to 1/10th of a cent.
         :param right:          The right of an option.
         :param date_range:     The dates to fetch.
-        :param interval_size:        The interval size in milliseconds. Applicable only to OHLC & QUOTE requests.
+        :param interval_size:  The interval size in milliseconds. Applicable only to OHLC & QUOTE requests.
+        :param use_rth:         If true, timestamps prior to 09:30 EST and after 16:00 EST will be ignored.
         :param progress_bar:   Print a progress bar displaying download progress.
 
         :return:               The requested data as a pandas DataFrame.
@@ -170,7 +206,47 @@ class ThetaClient:
         end_fmt = _format_date(date_range.end)
 
         # send request
-        hist_msg = f"MSG_CODE={MessageType.HIST.value}&START_DATE={start_fmt}&END_DATE={end_fmt}&root={root}&exp={exp_fmt}&strike={strike}&right={right.value}&sec={SecType.OPTION.value}&req={req.value}&IVL={interval_size}\n"
+        hist_msg = f"MSG_CODE={MessageType.HIST.value}&START_DATE={start_fmt}&END_DATE={end_fmt}&root={root}&exp={exp_fmt}&strike={strike}&right={right.value}&sec={SecType.OPTION.value}&req={req.value}&rth={use_rth}&IVL={interval_size}\n"
+        self._server.sendall(hist_msg.encode("utf-8"))
+
+        # parse response header
+        header_data = self._server.recv(20)
+        header: Header = Header.parse(hist_msg, header_data)
+
+        # parse response body
+        body_data = self._recv(header.size, progress_bar=progress_bar)
+        body: DataFrame = TickBody.parse(hist_msg, header, body_data)
+        return body
+
+    def get_hist_stock(
+            self,
+            req: StockReqType,
+            root: str,
+            date_range: DateRange,
+            interval_size: int = 0,
+            use_rth: bool = True,
+            progress_bar: bool = False,
+    ) -> pd.DataFrame:
+        """
+        Get historical stock data.
+
+        :param req:            The request type.
+        :param root:           The root symbol.
+        :param date_range:     The dates to fetch.
+        :param interval_size:  The interval size in milliseconds. Applicable only to OHLC & QUOTE requests.
+        :param use_rth:         If true, timestamps prior to 09:30 EST and after 16:00 EST will be ignored.
+        :param progress_bar:   Print a progress bar displaying download progress.
+
+        :return:               The requested data as a pandas DataFrame.
+        :raises ResponseError: If the request failed.
+        """
+        assert self._server is not None, _NOT_CONNECTED_MSG
+        # format data
+        start_fmt = _format_date(date_range.start)
+        end_fmt = _format_date(date_range.end)
+
+        # send request
+        hist_msg = f"MSG_CODE={MessageType.HIST.value}&START_DATE={start_fmt}&END_DATE={end_fmt}&root={root}&sec={SecType.STOCK.value}&req={req.value}&rth={use_rth}&IVL={interval_size}\n"
         self._server.sendall(hist_msg.encode("utf-8"))
 
         # parse response header
