@@ -129,6 +129,7 @@ class MessageType(enum.Enum):
     INFO = 2
     METADATA = 3
     CONNECTED = 4
+    VERSION = 5
 
     # API communication
     PING = 100
@@ -219,6 +220,22 @@ class OptionReqType(enum.Enum):
     ALT_CALCS = 304
 
 
+@enum.unique
+class StockReqType(enum.Enum):
+    """Stock request type codes."""
+
+    # FREE
+    EOD = 1
+
+    # VALUE
+    QUOTE = 101
+    VOLUME = 102
+    OHLC = 104
+
+    # STANDARD
+    TRADE = 201
+
+
 @dataclass
 class DateRange:
     """Represents an inclusive date range."""
